c     Mode coupling by an isotropic or anisotropic 3-D structure
c       with an isotropic structure as reference 
c
c     The program has two versions:
c     a purely isotropic one:
c            in that case, include "coupliso.inc" in the include
c            "couplage.inc" and compile with the
c            subroutines in file "coupliso.f"
c     an anisotropic one (which can also handle the isotropic case
c             but in a non-effective way):
c            in that case, include "couplani.inc" in the include
c            "couplage.inc" and compile with the
c            subroutines in file "couplani.f'
c
c   Calculates the coupling at one period only
c
c   The eigenfunctions must be calculated in the reference
c   structure by the programs of Saito (Numerical algorithms,
c   ed. Doornbos, Academic press, 1988)
c   before running the present program
c
c   Units for input and output 
c     in3  : input with different information on the run
c     in4  : input of different elements related to Saito:
c            the refererence structure and the modes
c               read in "filename"
c     in2  : input of the heterogenous structure
c               read in "filename"
c     in8  : output with different information on the run
c     in9  : main output with 3-D wavefield
c
c............................................................
c     include file contains arrays dimensions and the different
c                    commons of the program 
c*********************************************************************
c     NOTE THAT "COUPLAGE.INC" MUST ITSELF INCLUDE EITHER 
c     "COUPLISO.INC" OR "COUPLANI.INC" DEPENDING ON YOUR APPLICATION
      include "couplage.inc"
c*********************************************************************
c
c............................................................
      character*80 infile,outfile,outfile2
      character*68 filename
      character*15 line
      integer*4 narg
      pi = 4.*atan(1.)
c
      in3 = 3
      in4 = 4
      in2 = 2
      in8 = 8
      in9 = 9
      narg = iargc()
      if (narg.eq.3) then
         call getarg(1,infile)
         call getarg(2,outfile)
         call getarg(3,outfile2)
      else
         write(*,*)'Usage:  couplani in_file  out_file out_plot/
     &                   or coupliso in_file  out_file out_plot'
         stop
      endif
      open(in3,file=infile,status='old')
      open(in8,file=outfile,status='unknown')
      open(in9,file=outfile2,status='unknown')

c............................................................
c      reading elements concerning the reference model   
c
c        indepm=0 to use top of water layer as reference for depth coordinate
c               any other value to start at the bottom of the water layer
c                   (if there is any water...)
c
      read (in3,'(a)') filename                          
      read (in3,*)
      read (in3,*) indepm
      if (indepm.eq.0) 
     &       write (in8,'("reference model (water kept): ",a)') filename 
      if (indepm.ne.0) 
     &       write (in8,'("reference model (water stripped): ",a)') filename 
      open (in4,file=filename,status='old',err=999)
      call readmd 
      close (in4)
c
c......reading the period of the modes
c
      read (in3,*)
      read (in3,*)  freq0
      write (in8,'("frequency :",f6.3)') freq0
      perio=1./freq0
c
c       reading total number of Rayleigh and Love modes 
c
      read (in3,*)
      read (in3,*)  nmr,nml
      write(in8,'(i2," mode Rayleigh",3x,i2," mode Love")')nmr,nml

      nmm=nmr+nml

      if (nmm.gt.nmodes)      stop ' too many modes '
c
      read (in3,*)
c
c..........reading the Rayleigh and Love eigenfunctions 
      if (nmr.ne.0)   then
        read (in3,'(a)') filename                          
        write (in8,'("normal modes Ray   :",a)') filename 
        open (in4,file=filename,status='old',err=999)
        call readyp (1)
        close (in4)
      endif
c
      if (nml.ne.0)    then
        read (in3,'(a)') filename                          
        write (in8,'("normal modes Love  :",a)') filename 
        open (in4,file=filename,status='old',err=999)
        call readyp (2)
        close (in4)
      endif
c
c..... reading the heterogeneous model
c    hmin is the depth at which the heterogeneous model starts
c       measured from the surface of the reference model.
c       in case of water layer, the depth is measured from the
c       top of the water layer if indepm=0, from the bottom if not.
c    celbet defines how the model is paramterised; see coupliso and
c       couplani for details
c    delbet is a real which can be used to modify the overall
c       amplitude of the heterogeneity without making a new model file
c       for example chaging sign by '-1'. Default value for delbet: 1.
c    delani: the same as delbet for the anisotropic part of the model
c       for models having separate isotropic and anisotropic heterogeneities
      read (in3,*)
      read (in3,*) hmin
      read (in3,*)
      read (in3,'(a)') line
      read (line,'(a,1x,f5.2)') celbet,delbet
      if (celbet.eq.'ahi') read (line,'(a,1x,f5.2,1x,f5.2)') 
     &                                      celbet,delbet,delani
      if (celbet.eq.'ani'.or.celbet.eq.'anh') delani=delbet
      read (in3,*)
      write(in8,'(" prof and amplitude of hetero",f5.1,f10.4,2x,a)') 
     &                               hmin,delbet,celbet
      if (celbet.eq.'ani'.or.celbet.eq.'anh'.or.celbet.eq.'ahi')
     & write(in8,'(" amplitude aniso",f10.4)') delani
      read (in3,'(a)') filename
      write (in8,'("heterogeneous model:",a)') filename 
      open (in2,file=filename,status='old',err=999)
      call hetero (in2)
      close (in2)
c
      write (in9,'(i4,e12.3)') nnx,ddx
      write (in9,'(i4,e12.3)') nny,ddy
c
c ...   reading the index of the initial mode and write
c       out the surface displacements in the reference model
      read (in3,*)
      read (in3,*) iminit
      write (in8,'("iminit",i5)') iminit
      if (iminit.le.nmr) then
        write (in9,'(1p3e12.4)')
     &    abs(depli(2,iminit)),abs(depli(2,iminit)),abs(depli(1,iminit))
      else
        write (in9,'(1p3e12.4)')
     &    abs(depli(3,iminit)),abs(depli(3,iminit)),abs(depli(3,iminit))
      endif

c ...  calculating the depth integrals
      do ix=1,nnx
        do iy=1,nny
          call eigint(ix,iy)
        enddo
      enddo
c     write (*,*) 'depth integrals done'
c
c ... iterating and calculating the surface integrals
      read (in3,*)
      read (in3,*) nter
      call fredho (nter)

      close (in3)
      close (in8)
      close (in9)
      stop
 999  write (in8,'(a)') ' problem in opening file  ',filename
      stop
 900  format(i6,' points en p; erreur:',e12.4)
 901  format (f8.2,6(f8.3,' (',f6.3,')(#',f6.3,'),'))
 902  format (7(1x,f8.3))
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine fredho (nter)
c     performs the fredholm intregation
c
      include 'couplage.inc'
      complex cintff
      complex makefs

c...  initialise the Hankel functions and its integrated value
c            at (x,y) = (x',y')
      call inithk

c initialises dphi to the values of phi in the homogeneous model
      do ix=1,nnx
        do iy=1,nny
          do im=1,nmm
            phitot(ix,iy,im) = czero
            dphi(ix,iy,im) = czero 
          enddo
          phitot(ix,iy,iminit) = cone 
          dphi(ix,iy,iminit) = cone  
        enddo
      enddo
c
c iterates for the calculation of phi and dphi
      write (in9,*) nter
      do iter=1,nter

c calculates a new delta phi for mode m at point (x,y)
        do ix=1,nnx
          do iy=1,nny
            do im=1,nmm
              dphinw(ix,iy,im) = czero
c
c .............coupling to mode n
              do in=1,nmm
c                      numerical integration
                call filonx (ix,iy,im,in,cintff)
c               write (in8,*) 'regular',cintff
c                      singular integration
                cintff= cintff + makefs(ix,iy,im,in)
c               write (in8,*) '+ singu', cintff
                dphinw(ix,iy,im) = dphinw(ix,iy,im) -ydi*
     &             cexp(ydi*(ak(im)-ak(in))*(ix-1)*ddx)*cintff
              enddo
c             write (in8,*) ix,iy,im ,dphinw(ix,iy,im)   
            enddo
          enddo
        enddo

        do ix=1,nnx
          do iy=1,nny
            do im=1,nmm
              dphi(ix,iy,im) = dphinw(ix,iy,im)
              phitot(ix,iy,im) = phitot(ix,iy,im)  + dphi(ix,iy,im)
            enddo
          enddo
        enddo

c calculate the surface displacements
        call surfdp

        write (in8,*) 'finished iteration ',iter

      enddo

      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine surfdp
c
c  calculate the 3 components of the displacement at the surface
c
      include 'couplage.inc'
      external phase
      complex utot(3,nptx,npty)
      complex u1,u2,u3
      complex phi,phix,phiy
      complex ctemp

c calculate the 3-component displacements
c
c initialise the total displacement
      do ix=1,nnx
        do iy=1,nny
          utot(1,ix,iy)=czero
          utot(2,ix,iy)=czero
          utot(3,ix,iy)=czero
        enddo
      enddo
 
      if (nmm.eq.1) then
        write (in9,*) nmm
      else
        write (in9,*) nmm+1
      endif
c
c calculate the contributions from the individual modes
c Rayleigh  modes
      if (nmr.eq.0) goto 20
      do im=1,nmr
        akm=ak(im)
        um=depli(1,im)
        vm=depli(2,im)
        do ix=1,nnx
          phax=-akm*(ix-1)*ddx 
          do iy=1,nny
            call deriv1 (ix,iy,im,phi,phix,phiy)
            u1=-vm/akm*phix
            u2=-vm/akm*phiy
            u3=um*phi
c            calculate amplitudes, phase of vertical and differencial phases
            amp1=cabs(u1)
            pha1=phase(u1) 
            amp2=cabs(u2)
            pha2=phase(u2) 
            amp3=cabs(u3)
            pha3=phase(u3) 
            pha13=pha1-pha3
            pha21=pha2-pha1
            write (in9,'(1p7e12.4)')
     &             amp1,amp2,amp3,pha3,pha13,pha21,pha3+phax
c             increment total displacement
            utot(1,ix,iy)=utot(1,ix,iy)+u1*exp(ydi*phax)
            utot(2,ix,iy)=utot(2,ix,iy)+u2*exp(ydi*phax)
            utot(3,ix,iy)=utot(3,ix,iy)+u3*exp(ydi*phax)
          enddo
        enddo
      enddo
 20   continue

c Love modes
      if (nmr.eq.nmm) goto 29
      do im=nmr+1,nmm
        akm=ak(im)
        wm=depli(3,im)
        do ix=1,nnx
          phax=-akm*(ix-1)*ddx 
          do iy=1,nny
            call deriv1 (ix,iy,im,phi,phix,phiy)
            u1= wm/akm*phiy
            u2=-wm/akm*phix
            u3=czero 
c            calculate amplitudes, phase of vertical and differencial phases
            amp1=cabs(u1)
            pha1=phase(u1) 
            amp2=cabs(u2)
            pha2=phase(u2) 
            pha21=pha2-pha1
            write (in9,
     &          '(1pe12.4,1pe12.4,f3.0,1pe12.4,f3.0,1pe12.4,1pe12.4)')
     &              amp1,amp2,0.,pha2,0.,pha21,pha2+phax
c             increment total displacement
            utot(1,ix,iy)=utot(1,ix,iy)+u1*exp(ydi*phax)
            utot(2,ix,iy)=utot(2,ix,iy)+u2*exp(ydi*phax)
          enddo
        enddo
      enddo
 29   continue

c  analyse the total displacement (if several modes)
c  the total displacement contains the propagation phase, but
c  one has to correct from the dominant phase before going into the
c  function "phase" and add the phase again afterwards in order to 
c  have the correct number of 2*pi to plot the wavefronts
      if (nmm.eq.1) goto 99
      do ix=1,nnx
        phax=-ak(iminit)*(ix-1)*ddx 
        do iy=1,nny
          amp1=cabs(utot(1,ix,iy))
          amp2=cabs(utot(2,ix,iy))
          amp3=cabs(utot(3,ix,iy))
          ctemp = utot(1,ix,iy)*exp(-ydi*phax) 
          pha1=phase(ctemp) 
          ctemp = utot(2,ix,iy)*exp(-ydi*phax) 
          pha2=phase(ctemp)
          ctemp = utot(3,ix,iy)*exp(-ydi*phax) 
          pha3=phase(ctemp)
          if (iminit.le.nmr) then
            pha0=pha3
            pha4=pha1-pha3
            pha5=pha2-pha1
          else
            pha0=pha2
            pha4=pha3-pha2
            pha5=pha2-pha1
          endif
          write (in9,'(7e12.3)')amp1,amp2,amp3,pha0,pha4,pha5,pha0+phax
        enddo
      enddo

 99   continue
      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      function phase(z)
c calculates the phase of a complex number
      complex z
      a1=real(z)
      a2=imag(z)
      if (a1.eq.0..and.a2.eq.0.) then
        phase=0.
      else
        phase=atan2(a2,a1)
      endif
      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine inithk
c     initialises different values of the Hankel function needed
c     in the integration, and the mostly analytical integration
c     of the Hankel function*exp(ikx) close to the scattering point
c
      include 'couplage.inc'
      real rzero,rone,rtwo,rthr
      complex chank(4)
      real bess0,bess1,bess2,bess3
      complex sint00(nmodes,nmodes),sint01(nmodes,nmodes)
      complex sint02(nmodes,nmodes)

      rzero=0.
      rone=1.
      rtwo=2.
      rthr=3.

c...  initialise the Hankel functions
c          Hankel functions to be used in the Filon integrals
      do ix=1,nnx
        xx=(ix-1)*ddx
        do iy=1,nny
          yy=(iy-1)*ddy
          if (ix.eq.1.and.iy.eq.1) goto 10
          rr=sqrt(xx*xx+yy*yy)
          do im=1,nmm
            dtemp=ak(im)*rr

            call bessjy(dtemp,rzero,bess0,webe0,tmp1,tmp2)
            chank(1)=cmplx(bess0,-webe0)
            call bessjy(dtemp,rone ,bess1,webe1,tmp1,tmp2)
            chank(2)=cmplx(bess1,-webe1)
            call bessjy(dtemp,rtwo ,bess2,webe2,tmp1,tmp2)
            chank(3)=cmplx(bess2,-webe2)

            hank0(im,ix,iy)=chank(1)*cexp(ydi*ak(im)*rr)
            hank1(im,ix,iy)=chank(2)*cexp(ydi*ak(im)*rr)
            hank2(im,ix,iy)=chank(3)*cexp(ydi*ak(im)*rr)
          enddo
c         write (in8,'(2i3,f6.0,6e12.3)') ix,iy,rr,hank0(1,ix,iy),
c    &              hank1(1,ix,iy),hank2(1,ix,iy)
 10       continue
        enddo
      enddo

c  Hankel functions integrated over dx*dy close to the singularity
      do im=1,nmm
       do in=1,nmm
        akm=ak(im)
        akn=ak(in)
        astep=0.01
        akk=amax1(akm,akn)
        nstep=int(akk*ddx/astep)
        write (in8,*) 'step in Hankel function',nstep
        delx=ddx/nstep
        dely=ddy/nstep
        sint0(im,in)=czero
        sint1(im,in)=czero
        sint2(im,in)=czero
c
c...   integration over the 4 corners
        do ix=1,nstep
          xx = delx*(ix-0.5)
          do iy=1,nstep
            yy = dely*(iy-0.5)
            rr = sqrt (xx*xx+yy*yy)
              if (rr.ge.ddx) then
            dtemp=akm*rr

            call bessjy(dtemp,rzero,bess0,webe0,tmp1,tmp2)
            chank(1)=cmplx(bess0,-webe0)
            call bessjy(dtemp,rone ,bess1,webe1,tmp1,tmp2)
            chank(2)=cmplx(bess1,-webe1)
            call bessjy(dtemp,rtwo ,bess2,webe2,tmp1,tmp2)
            chank(3)=cmplx(bess2,-webe2)

            cosfi=-xx/rr
            cos2fi=(xx*xx-yy*yy)/(rr*rr)
            sint0(im,in)=sint0(im,in)+ chank(1)*cos(akn*xx)
            sint1(im,in)=sint1(im,in)+ chank(2)*(-ydi)*sin(akn*xx)*cosfi
            sint2(im,in)=sint2(im,in)+ chank(3)*cos(akn*xx)*cos2fi
              endif
          enddo
        enddo
        sint0(im,in)=4.*sint0(im,in)*delx*dely
        sint1(im,in)=4.*sint1(im,in)*delx*dely
        sint2(im,in)=4.*sint2(im,in)*delx*dely

c   add the analytical part calculated over the inner circle 
        dtemp=akm*ddx

        call bessjy(dtemp,rzero,bess0,webe0,tmp1,tmp2)
        chank(1)=cmplx(bess0,-webe0)
        call bessjy(dtemp,rone ,bess1,webe1,tmp1,tmp2)
        chank(2)=cmplx(bess1,-webe1)
        call bessjy(dtemp,rtwo ,bess2,webe2,tmp1,tmp2)
        chank(3)=cmplx(bess2,-webe2)
        call bessjy(dtemp,rthr ,bess3,webe3,tmp1,tmp2)
        chank(4)=cmplx(bess3,-webe3)
c
        dtemp=akn*ddx
        call bessjy(dtemp,rzero,bess0,tmp1,tmp2,tmp3)
        call bessjy(dtemp,rone ,bess1,tmp1,tmp2,tmp3)
        call bessjy(dtemp,rtwo ,bess2,tmp1,tmp2,tmp3)
        call bessjy(dtemp,rthr ,bess3,tmp1,tmp2,tmp3)

        if (im.eq.in) then
        sint00(im,in)=pi*ddx*ddx*(bess0*chank(1) + bess1*chank(2))
        sint01(im,in)=pi*ddx*ddx*(bess1*chank(2) - bess0*chank(3))
        sint01(im,in)=4./akn/akn + sint01(im,in)/ydi
        sint01(im,in)=-sint01(im,in)
        sint02(im,in)=pi*ddx*ddx*(bess2*chank(3) - bess1*chank(4))
        sint02(im,in)=-ydi*8./akn/akn - sint02(im,in)
        else
        sint00(im,in)=akn*bess1*chank(1)-akm*bess0*chank(2)
        sint00(im,in)=4.*ydi*(1.+ddx*pi*sint00(im,in)/2./ydi)
        sint00(im,in)=sint00(im,in)/(akn*akn-akm*akm)
c
        sint01(im,in)=akn*bess2*chank(2)-akm*bess1*chank(3)
        sint01(im,in)=ddx*pi*sint01(im,in)/2./ydi
        sint01(im,in)=4.*(akn/akm + sint01(im,in))/(akn*akn-akm*akm)
        sint01(im,in)=-sint01(im,in)
c
        sint02(im,in)=akn*bess3*chank(3)-akm*bess2*chank(4)
        sint02(im,in)=-(akn/akm)**2 + ydi*ddx*pi*sint02(im,in)/2.
        sint02(im,in)=4.*ydi*sint02(im,in)/(akn*akn-akm*akm)
c
        endif
c       write (in8,*) im,in,sint00(im,in),sint0(im,in)
c       write (in8,*) sint01(im,in),sint1(im,in)
c       write (in8,*) sint02(im,in),sint2(im,in)

        sint0(im,in)=sint0(im,in)+sint00(im,in)
        sint1(im,in)=sint1(im,in)+sint01(im,in)
        sint2(im,in)=sint2(im,in)+sint02(im,in)

       enddo
      enddo

      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine filonx (ix,iy,im,in,cintf)
c      drives the calculation of the filon integrations in x
c           around the point (ix,iy)
c      performs first two filon integration in r to prepare
c      the integrand for the integration in x away from line iy2=iy
c      integrates in y with simpson rule around line iy2=iy
c      and integrates in x with Filon on both sides of the point ix2=ix

      include 'couplage.inc'
      complex cakn,cakm,cak
      real dx(nptx),rr(npty)
      complex ff(npty),ffi(nptx)
      complex cintfy,cintf
      complex makeff,filon

      cakn = -ydi*ak(in)
      cakm = -ydi*ak(im)
c 
c    double Filon integration away from line iy2=iy
c
      do ix2=1,nnx   
        dx(ix2) = (ix2-ix)*ddx
        ffi(ix2)=czero
        if (iy.gt.2) then 
          ny1=iy-1
          do iy2=1,ny1  
            dy = (iy2-iy)*ddy
            rr(iy2)=sqrt(dx(ix2)*dx(ix2)+dy*dy)
            ff(iy2)= makeff (1,ix,iy,ix2,iy2,im,in)
          enddo
          ffi(ix2)=ffi(ix2)+ filon(1,ny1,rr,ff,cakm)
        endif 
        if (iy.lt.nny-1) then
          ny1=iy+1
          do iy2=ny1,nny
            dy = (iy2-iy)*ddy
            rr(iy2)=sqrt(dx(ix2)*dx(ix2)+dy*dy)
            ff(iy2)= makeff (1,ix,iy,ix2,iy2,im,in)
          enddo
          ffi(ix2)= ffi(ix2) + filon (ny1,nny,rr,ff,cakm)
        endif
      enddo
      cintf= filon (1,nnx,dx,ffi,cakn)
c 
c   2 Simpson + Filon integration close to line iy2=iy
c
      do iter=1,2
        if (iter.eq.1.and.ix.le.2) goto 10
        if (iter.eq.2.and.ix.ge.nnx-1) goto 10
        if (iter.eq.1) then
          nx1=1
          nx2=ix-1
          signe=1.
          cak=cakn-cakm
        else
          nx1=ix+1
          nx2=nnx
          signe=-1.
          cak=cakn+cakm
        endif
        do ix2=nx1,nx2 
          dx(ix2) = (ix2-ix)*ddx
          if (iy.ne.1) ff(iy-1)= makeff (2,ix,iy,ix2,iy-1,im,in)
          ff(iy)= makeff (2,ix,iy,ix2,iy,im,in)
          if (iy.ne.nny) ff(iy+1)= makeff (2,ix,iy,ix2,iy+1,im,in)
          if (iy.ne.1.and.iy.ne.nny) then
            cintfy = (ff(iy-1)+4.*ff(iy)+ff(iy+1))*ddy/3.
          else
            if (iy.eq.1) cintfy = (ff(iy)+ff(iy+1))*ddy/2.
            if (iy.eq.nny) cintfy = (ff(iy-1)+ff(iy))*ddy/2.
          endif
          ffi(ix2)=cintfy*cexp(signe*cakm*dx(ix2))
c         write (in8,*) 'cintfy',nx1,nx2,cintfy
        enddo
        cintf=cintf+ filon (nx1,nx2,dx,ffi,cak)
 10   continue
      enddo
c 
      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      complex function filon (nn1,nn2,pp,ff,calpha)
c   calculates the integral of ff*exp(calpha*pp)
c   from point nn1 to point nn2
c   using the Filon intergation scheme
c     ff is a complex function, pp is a real function
c
      real pp(*)
      complex ff(*)
      complex calpha,cint
    
      cint =cmplx(0.,0.)

      ddp=(pp(nn2)-pp(nn1))/(nn2-nn1)
      test=cabs(calpha*ddp)

      if (test.gt.0.01) then
c
c ....Filon integral in the case "alpha" non-zero
c
        do j= nn1,nn2-1
          cint= cint + (ff(j+1)-ff(j))*
     &       (cexp(calpha*pp(j+1))-cexp(calpha*pp(j)))/(pp(j+1)-pp(j))
        enddo
        cint=-cint/calpha
c  ... contributions from end-points
        cint= cint + 
     &    ff(nn2)*exp(calpha*pp(nn2)) - ff(nn1)*exp(calpha*pp(nn1))
        cint = cint/calpha
c
c.... ordinary integration in the case "alpha' small 
c
      else
        cint= cint + ff(nn1)*cexp(calpha*pp(nn1))*(pp(nn1+1)-pp(nn1))/2.
        do j= nn1+1,nn2-1
          cint= cint + ff(j)*cexp(calpha*pp(j))*(pp(j+1)-pp(j-1))/2.
        enddo
        cint= cint + ff(nn2)*cexp(calpha*pp(nn2))*(pp(nn2)-pp(nn2-1))/2.

      endif

      filon=cint

      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine deriv (ix2,iy2,in,phi,phix,phiy,phixx,phixy,phiyy)

c... prepares the spatial derivatives of dphi for mode in
c             at point (ix2,iy2)
c atttention: phixx is not the total derivative (term in k**2 omitted)

      include 'couplage.inc'
      complex phi,phix,phiy,phixx,phixy,phiyy
      complex phix0

      akn=ak(in)

      phi=dphi(ix2,iy2,in)

      if (iy2.ne.1.and.iy2.ne.nny) then
        phiy = (dphi(ix2,iy2+1,in)-dphi(ix2,iy2-1,in))/2./ddy
      else
        if (iy2.eq.1) phiy=(dphi(ix2,iy2+1,in)-dphi(ix2,iy2,in))/ddy
        if (iy2.eq.nnx) phiy=(dphi(ix2,iy2,in)-dphi(ix2,iy2-1,in))/ddy
      endif

      if (ix2.ne.1.and.ix2.ne.nnx) then
        phix0 = (dphi(ix2+1,iy2,in)-dphi(ix2-1,iy2,in))/2./ddx
      else
        if (ix2.eq.1) phix0 = (dphi(ix2+1,iy2,in)-dphi(ix2,iy2,in))/ddx
        if (ix2.eq.nnx) phix0=(dphi(ix2,iy2,in)-dphi(ix2-1,iy2,in))/ddx
      endif
      phix= phix0 -ydi*akn*dphi(ix2,iy2,in)

      if (ix2.gt.2.and.ix2.lt.nnx-1) then
        phixx =  (-dphi(ix2+2,iy2,in)-dphi(ix2-2,iy2,in)
     &   +16.*dphi(ix2+1,iy2,in)+16.*dphi(ix2-1,iy2,in)
     &   -30.*dphi(ix2,iy2,in)) /(12.*ddx*ddx)
      else
        if (ix2.eq.1.or.ix2.eq.nnx) phixx = czero
        if (ix2.eq.2.or.ix2.eq.nnx-1) phixx = (dphi(ix2+1,iy2,in)
     &     -2.*dphi(ix2,iy2,in)+dphi(ix2-1,iy2,in))/ddx/ddx
      endif
      phixx =  phixx -2.*ydi*akn*phix0

      if (iy2.gt.2.and.iy2.lt.nny-1) then
        phiyy = (-dphi(ix2,iy2+2,in)-dphi(ix2,iy2-2,in)
     &   +16.*dphi(ix2,iy2+1,in)+16.*dphi(ix2,iy2-1,in)
     &   -30.*dphi(ix2,iy2,in)) /(12.*ddy*ddy)
      else
        if (iy2.eq.1.or.iy2.eq.nny) phiyy=czero
        if (iy2.eq.2.or.iy2.eq.nny-1) phiyy =
     &       (dphi(ix2,iy2+1,in)-2.*dphi(ix2,iy2,in)+
     &              dphi(ix2,iy2-1,in))/ddy/ddy
      endif
     
      j1x=ix2-1
      j2x=ix2+1
      j1y=iy2-1
      j2y=iy2+1
      temp=1.
      if (j1x.eq.0) then
        j1x=1
        temp=temp*2.
      endif
      if (j2x.eq.nnx+1) then
        j2x=nnx
        temp=temp*2.
      endif
      if (j1y.eq.0) then
        j1y=1
        temp=temp*2.
      endif
      if (j2y.eq.nny+1) then
        j2y=nny
        temp=temp*2.
      endif
      phixy =-ydi*akn*phiy + temp*
     &  (dphi(j2x,j2y,in)-dphi(j2x,j1y,in)-dphi(j1x,j2y,in)
     &   +dphi(j1x,j1y,in)) /(4.*ddx*ddy)
      
      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine deriv1 (ix2,iy2,in,phi,phix,phiy)

c... prepares the spatial derivatives of phitot for mode in
c             at point (ix2,iy2)

      include 'couplage.inc'
      complex phi,phix,phiy

      akn=ak(in)

      phi=phitot(ix2,iy2,in)

      if (iy2.ne.1.and.iy2.ne.nny) then
        phiy = (phitot(ix2,iy2+1,in)-phitot(ix2,iy2-1,in))/2./ddy
      else
        if (iy2.eq.1) 
     &       phiy=(phitot(ix2,iy2+1,in)-phitot(ix2,iy2,in))/ddy
        if (iy2.eq.nnx) 
     &       phiy=(phitot(ix2,iy2,in)-phitot(ix2,iy2-1,in))/ddy
      endif

      if (ix2.ne.1.and.ix2.ne.nnx) then
        phix = (phitot(ix2+1,iy2,in)-phitot(ix2-1,iy2,in))/2./ddx
      else
        if (ix2.eq.1) 
     &       phix = (phitot(ix2+1,iy2,in)-phitot(ix2,iy2,in))/ddx
        if (ix2.eq.nnx) 
     &       phix=(phitot(ix2,iy2,in)-phitot(ix2-1,iy2,in))/ddx
      endif
      phix= phix -ydi*akn*phitot(ix2,iy2,in)

      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine readyp (ittt)
c
c  read the modes calculated by Saito's programs 
c       this version of readyp is for one period only
c       and for proper modes only 
c
c       ittt  = 1 for Rayleigh waves
c       ittt  = 2 for Love waves
c       
      include "couplage.inc"
c
c
      dimension dep(nptz)
      character ligne*80
c..
      in4 = 4
c
      if (ittt.eq.1) im0 = 0
      if (ittt.eq.2) im0 = nmr
c
      im=im0
 30   continue
      im = im+1

 42   read (in4,'(a)',end=99) ligne
      if (ligne(6:10).eq.'DEPTH')  then    
        read (ligne,'(26x,i4)') ip
        goto 42
      endif
      if (ligne(21:22).eq.'??')    goto 42
      if (ligne(15:15).ne.'T'.and.ligne(7:7).ne.'T')  goto 42
c
c.........Rayleigh modes
      if (ittt .eq.1)    then
c        read (in4,'(8x,3f12.5///20x,f12.5//////31x,e12.5
         read (in4,'(   4f12.5///22x,e12.5//////31x,e12.5,//)',end=99)
     &                 ttt,vp,vg,vg2,ec,atte
         atte=0.
         write (in8,*) im,ttt,vp
         akk = 2.*3.14159/vp/ttt
         ak(im) = akk
         aq(im) = -akk*atte/2.
c        anorm = sqrt(ec*vp*ttt/3.14159/2.)
         anorm = sqrt(2.*ec)/akk
         i=0
         depsave=-1.
         do 43 ilu = 1,ip
           read (in4,'(i5,1p5e15.6,0p)',end=99)
     &         ii,deplu ,yy1lu ,yy2lu ,yy3lu ,yy4lu 
c.         skip values in water if requested
           if ((epm.ne.0.and.indepm.ne.0).and.yy4lu.eq.0.) then
             if (deplu.ne.depsave)  then
               depsave=deplu
               goto 43
             endif
           endif
           i=i+1
           dep(i)=deplu
c             vertical  component
           dplz(1,i,im)=yy1lu/anorm
c            horizontal component
           dplz(2,i,im)=yy3lu/anorm
           dplz(3,i,im)=0.
           dplz(4,i,im)=-(yy2lu+akk*af(i)*yy3lu)/aa(i)/anorm
           if (al(i).eq.0.) then
             dplz(5,i,im)=-akk*yy1lu/anorm
           else
             dplz(5,i,im)=(akk*yy1lu-yy4lu/al(i))/anorm
           endif
           dplz(6,i,im)=0.
           if(ii.eq.ip) goto 44
 43      continue
 44      continue
         nnmd=i 
         if (nnmd.gt.nptz) stop 'pb with dim of PSV eigenfunctions'
c
c.......eigenfunctions at the surface of the solid
           depli(1,im) = dplz(1,1,im) 
           depli(2,im) = dplz(2,1,im) 
           depli(3,im) = 0.   
           if (epm.ne.0..and.indepm.eq.0) then
c            if there is a non-zero water layer which has
c            been kept in the model, we must take the
c            eigenfunctions at the base of the water layer
             depsave=-1.
             i=1
             dowhile (dep(i).ne.depsave) 
               depsave=dep(i)
               i=i+1
             enddo
             depli(1,im) = dplz(1,i,im) 
             depli(2,im) = dplz(2,i,im) 
             depli(3,im) = 0.   
           endif
 45      continue
         write (in8,*) im,'depli',i,depli(1,im),depli(2,im)

      endif
c
c.........Love modes
      if (ittt .eq.2)    then
c        read (in4,'(8x,3f12.5///8x,f12.5//////31x,e12.5
c    &             //)',end=99)
c    &                 ttt,vp,vg,ec,atte
         read (in4,'(   3f12.5///22x,1PE12.5/////)',end=99)
     &                 ttt,vp,vg,ec
         read (in4,'(31x,1PE12.5//)',end=99) atte
         atte=0.
         write (in8,*) im,ttt,vp
         akk = 2.*3.14159/vp/ttt  
         ak(im) = akk
         aq(im) = -akk*atte/2.
c        anorm = sqrt(ec*vp*ttt  /3.14159/2.)
         anorm = sqrt(2.*ec)/akk
         i=0
         depsave=-1.
c
c...........read the eigenfunctions
         do 63 ilu = 1,ip
           read (in4,'(i5,1p3e15.6,0p)',end=99)
     &         ii,deplu,yy1,yy2
c.         skip values in water if requested
           if ((epm.ne.0..and.indepm.ne.0).and.yy2.eq.0.) then
             if (deplu.ne.depsave)  then
               depsave=deplu
               goto 63
             endif
           endif
           i=i+1
           dep(i)=deplu
           dplz(1,i,im)=0.
           dplz(2,i,im)=0.
           dplz(3,i,im)=yy1/anorm
           dplz(4,i,im)=0.
           dplz(5,i,im)=0.
           if (al(i).eq.0.) then
             dplz(6,i,im)=0.
           else
             dplz(6,i,im)=-yy2/al(i)/anorm
           endif
           if (ii.eq.ip)  goto 64
 63      continue
 64      continue
         nnmd=i
         if (nnmd.gt.nptz) stop 'pb with dim of SH eigenfunctions'
         
c
c.......eigenfunctions at the surface of the solid
           depli(1,im) = 0.        
           depli(2,im) = 0.        
           depli(3,im) = dplz(3,1,im) 
           if (epm.ne.0..and.indepm.eq.0) then
c            if there is a non-zero water layer which has
c            been kept in the model, we must take the
c            eigenfunctions at the base of the water layer
             depsave=-1.
             i=1
             dowhile (dep(i).ne.depsave) 
               depsave=dep(i)
               i=i+1
             enddo
             depli(1,im) = 0.        
             depli(2,im) = 0.        
             depli(3,im) = dplz(3,i,im) 
           endif
 65      continue
         write (in8,*) im,'depli',i,depli(3,im)

      endif
c
      if (ittt.eq.1.and.im.lt.nmr)     goto 30
      if (ittt.eq.2.and.im.lt.(nmm))   goto 30

      return
c
 99   continue
      write (in8,*) im, ' some modes missing ?'
      stop
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine readmd 
c  reads the reference model used to calculate the eigenfunctions
c  and resamples it at the same depth points as the eigenfunctions
c
      include "couplage.inc"
c       
      
      dimension alp(2*nptz),bet(2*nptz),rho(2*nptz)
      dimension ep(2*nptz),dep(2*nptz)
c
      CHARACTER TITR*60
c
      in4 = 4
c
      read (in4,'(1x,a)') titr
c     write (in8,'(a)') titr
c
      read (in4,*) nnmd
      if (nnmd.gt.2*nptz)  stop 'not enough dimension for model saito'
      read (in4,*)
c
      dep(1) = 0.
      DO 10 i=1,nnmd
        read (in4,*,end=11) ep(i),rho(i),alp(i),bet(i)
        dep(i+1) = dep(i) + ep(i)
 10   CONTINUE
 11   CONTINUE
c
c......  calculate the thickness of the water layer 
c
      epm = 0.
      iim = 0 
      if (bet(1).eq.0.)  then
        do 14 i =1 ,nnmd
          iim=i
          if (ep(i).eq.0.)  goto 15
          epm = epm + ep(i)
 14     continue
 15     continue
      endif
c
C     in all points:
c..........A F and L at the depths where eigenfunctions are known)
c       (one does not keep the values in the water layer if
c        indepm.ne.0)
c
c       reset initial point a the surface if indepm.eq.0
      if (indepm.eq.0) iim=0

      J = 1
      I = IIM+1
c..    pointm (j,i)
      aa(j) = rho(i)*alp(i)*alp(i)
      al(j) = rho(i)*bet(i)*bet(i)
      af(j) = (aa(j)-2.*al(j))
      ro(j) = rho(i)
c..
      hh(j) = epm
      if (indepm.eq.0) hh(j)=0.
      DO 20 J = 2,nnmd
      IF (ep(I).EQ.0.) THEN
        I = I+1
        IF (I.GT.nnmd) GOTO 21
c..     pointm (j,i)
        aa(j) = rho(i)*alp(i)*alp(i)
        al(j) = rho(i)*bet(i)*bet(i)
        af(j) = (aa(j)-2.*al(j))
        ro(j) = rho(i)
c..
        hh(j) = hh(j-1)
      ELSE
        I = I+2
        IF (I.GE.nnmd) GOTO 21
c..     pointm(j,i)
        aa(j) = rho(i)*alp(i)*alp(i)
        al(j) = rho(i)*bet(i)*bet(i)
        af(j) = (aa(j)-2.*al(j))
        ro(j) = rho(i)
c..
        hh(j) = hh(j-1) + ep(i-2)+ep(i-1)
      ENDIF
      nj = j
      if (nj.gt.nptz)  stop ' not enough dimension in nptz'
 20   CONTINUE
 21   CONTINUE
c
c     write (in8,*) epm
c     write (in8,*) nj
c     write (in8,*) '     A       F       L      ro'
c     do 30 j=1,nj
c     write(in8,'(4f8.3,f10.4)') aa(j),af(j),al(j),ro(j),hh(j)
c 30  continue
c
      return 
      end
