c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C  Subroutines which are specific to the isotropic case
c    in the mode coupling program couplage.f
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      complex function makeff (index,ix,iy,ix2,iy2,im,in)
c
c      calculates the integrand at the position ix2,iy2
c      of the integral in x,y in the non-singular case
c     normalises for the most general case if index=1
c     normalises for the area close to iy2=iy if index=2

      include 'couplage.inc'

      complex phi,phix,phiy,phixx,phixy,phiyy
      complex fcp
c
c...prepares the geometrical elements
      dx = (ix2-ix)*ddx
      dy = (iy2-iy)*ddy
      rr = sqrt (dx*dx + dy*dy)
      cosfi=-dx/rr
      sinfi=-dy/rr
      cos2fi=cosfi*cosfi-sinfi*sinfi
      sin2fi=2.*cosfi*sinfi
c... indexes to be used for the Hankel functions
      idx=iabs(ix2-ix)+1
      idy=iabs(iy2-iy)+1

c     if (ix.eq.15.and.ix2.eq.10) 
c    &                write (in8,*) 'posi',ix,iy,ix2,iy2,idx,idy

      akm=ak(im)
      akn=ak(in)

c... prepares the spatial derivatives of dphi
      call deriv (ix2,iy2,in,phi,phix,phiy,phixx,phixy,phiyy)
      
c....   Rayleigh to Rayleigh coupling
      if (im.le.nmr.and.in.le.nmr) then
        
        fcp = cp1(im,in,ix2,iy2)*hank2(im,idx,idy)*
     &        (cos2fi*(-akn*akn*phi+phixx-phiyy) 
     &         +2.*sin2fi*phixy)
        fcp = fcp + cp2(im,in,ix2,iy2)*hank1(im,idx,idy)*
     &        (cosfi*phix+sinfi*phiy)
        fcp = fcp + hank0(im,idx,idy)*
     &         (cp3(im,in,ix2,iy2)*(phixx+phiyy)
     &          +cp4(im,in,ix2,iy2)*phi)

      endif

c....   Love to Love coupling
      if (im.gt.nmr.and.in.gt.nmr) then
        
        fcp = cp1(im,in,ix2,iy2)*hank2(im,idx,idy)*
     &        (cos2fi*(-akn*akn*phi+phixx-phiyy) 
     &         +2.*sin2fi*phixy)
        fcp = fcp + cp2(im,in,ix2,iy2)*hank1(im,idx,idy)*
     &        (cosfi*phix+sinfi*phiy)
      endif

c....   Love to Rayleigh and Rayleigh to Love coupling
      if ((im.le.nmr.and.in.gt.nmr).or.(im.gt.nmr.and.in.le.nmr)) then
        
        fcp = cp1(im,in,ix2,iy2)*hank2(im,idx,idy)*
     &        (sin2fi*(-akn*akn*phi+phixx-phiyy) -
     &         2.*cos2fi*phixy)
        fcp = fcp + cp2(im,in,ix2,iy2)*hank1(im,idx,idy)*
     &        (cosfi*phiy-sinfi*phix)
      endif

c ... normalises for the change of variable from y to R
      if (index.eq.1) fcp = -fcp/sinfi

c ... removes a phase in case of iy2 close to iy
      if (index.eq.2) fcp = fcp*cexp(-ydi*akm*rr)

      makeff=fcp

      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      complex function makefs (ix,iy,im,in)
c
c      calculates the singular part of the integration

      include 'couplage.inc'

      complex fcp,hank0s,hank1s,hank2s
      complex phi,phix,phiy,phixx,phixy,phiyy
c
      ix2=ix
      iy2=iy

c     write (in8,*) 'selv',ix,iy

      akm=ak(im)
      akn=ak(in)

c...  replaces the hankel function by the integrated values
      hank0s=sint0(im,in)
      hank1s=sint1(im,in)
      hank2s=sint2(im,in)

c... prepares the spatial derivatives of dphi
      call deriv (ix2,iy2,in,phi,phix,phiy,phixx,phixy,phiyy)
      
c....   Rayleigh to Rayleigh coupling
      if (im.le.nmr.and.in.le.nmr) then
        
        fcp = cp1(im,in,ix2,iy2)*hank2s*
     &        (-akn*akn*phi+phixx-phiyy) 
        fcp = fcp + cp2(im,in,ix2,iy2)*hank1s*phix
        fcp = fcp + hank0s*
     &         (cp3(im,in,ix2,iy2)*(phixx+phiyy)
     &          +cp4(im,in,ix2,iy2)*phi)
      endif

c....   Love to Love coupling
      if (im.gt.nmr.and.in.gt.nmr) then
        
        fcp = cp1(im,in,ix2,iy2)*hank2s*
     &        (-akn*akn*phi+phixx-phiyy) 
        fcp = fcp + cp2(im,in,ix2,iy2)*hank1s*phix
      endif

c....   Love to Rayleigh and Rayleigh to Love coupling
      if ((im.le.nmr.and.in.gt.nmr).or.(im.gt.nmr.and.in.le.nmr)) then
        
        fcp = cp1(im,in,ix2,iy2)*hank2s*(-2.)*phixy
        fcp = fcp + cp2(im,in,ix2,iy2)*hank1s*phiy
      endif

c
c... reduces the size for points on the model boundaries
      if (ix.eq.1.or.ix.eq.nnx) fcp=fcp/2.
      if (iy.eq.1.or.iy.eq.nny) fcp=fcp/2.

      makefs=fcp

      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine eigint(ix,iy)
c
c       calculate les elementary integrals in z for the mode coupling 
c
      include "couplage.inc"

      real dmu(nptz),dla(nptz),dro(nptz)
c
c......calculate the anomalies in mu, lambda and rho at (ix,iy)
      call elcoef(ix,iy,dmu,dla,dro)


c........elements for Love-Rayleigh coupling
      if (nmr.eq.0.or.nmr.eq.nmm)    goto 21
      do 20 im = 1,nmr
       do 20 im2 = nmr+1,nmm
        temp1=0.
        temp2=0.
        do 22 j =ntop,nbot 
          temp1= temp1 - dmu(j)*dplz(2,j,im)*dplz(3,j,im2)*dz(j)
          temp2=temp2+(-dro(j)*dplz(2,j,im)*dplz(3,j,im2)/ak(im)/ak(im2)
     &    -dmu(j)*(dplz(1,j,im)-dplz(5,j,im)/ak(im))
     &               *dplz(6,j,im2)/ak(im2))
     &        * dz(j)
  22    continue
        cp1(im,im2,ix,iy) = temp1*ak(im)/ak(im2)
        cp1(im2,im,ix,iy) = -temp1*ak(im2)/ak(im) 
        cp2(im,im2,ix,iy) = temp2*ak(im)
        cp2(im2,im,ix,iy) = -temp2*ak(im2)
  20  continue
  21  continue
c
c........elements for Rayleigh-Rayleigh coupling
      if (nmr.eq.0)    goto 31
      do 30 im = 1,nmr
       do 30 im2 = im,nmr 
        temp1=0.
        temp2=0.
        temp3=0.
        temp3b=0.
        temp4=0.
        do 32 j =ntop,nbot
          temp1= temp1 - dmu(j)*dplz(2,j,im)*dplz(2,j,im2)*dz(j)
          temp2=temp2 +(dro(j)*dplz(2,j,im)*dplz(2,j,im2)/ak(im)/ak(im2)
     &     - dmu(j)*(dplz(1,j,im)-dplz(5,j,im)/ak(im))*
     &           (dplz(1,j,im2)-dplz(5,j,im2)/ak(im2)))
     &        * dz(j)
          temp3=temp3 +
     &        ((dmu(j)+dla(j))*dplz(2,j,im)*dplz(2,j,im2)*ak(im)/ak(im2)
     &     + dla(j)*dplz(4,j,im)*dplz(2,j,im2)/ak(im2))
     &        * dz(j)
          temp3b=temp3b+
     &        ((dmu(j)+dla(j))*dplz(2,j,im2)*dplz(2,j,im)*ak(im2)/ak(im)
     &     + dla(j)*dplz(4,j,im2)*dplz(2,j,im)/ak(im))
     &        * dz(j)
          temp4=temp4 +(dro(j)*dplz(1,j,im)*dplz(1,j,im2)
     &     - dla(j)*(dplz(2,j,im)*dplz(4,j,im2)*ak(im)+
     &           dplz(2,j,im2)*dplz(4,j,im)*ak(im2))
     &     - (dla(j)+dmu(j))*dplz(2,j,im)*dplz(2,j,im2)*ak(im)*ak(im2)
     &     - (dla(j)+2.*dmu(j))*dplz(4,j,im)*dplz(4,j,im2))
     &         * dz(j)
  32    continue
        cp1(im,im2,ix,iy) = temp1*ak(im)/ak(im2)
        cp1(im2,im,ix,iy) = temp1*ak(im2)/ak(im) 
        cp2(im,im2,ix,iy) = temp2*ak(im)
        cp2(im2,im,ix,iy) = temp2*ak(im2)
        cp3(im,im2,ix,iy) = temp3
        cp3(im2,im,ix,iy) = temp3b
        cp4(im,im2,ix,iy) = temp4
        cp4(im2,im,ix,iy) = temp4
        dc = -2.*(cp2(im,im2,ix,iy)/ak(im2)+cp1(im,im2,ix,iy)
     &              +cp4(im,im2,ix,iy)/ak(im2)/ak(im2))
  30  continue
  31  continue
c
c........elements for Love-Love coupling
      if (nmr.eq.nmm)    goto 41
      do 40 im = nmr+1,nmm
       do 40 im2 = im,nmm 
        temp1=0.
        temp2=0.
        do 42 j =ntop,nbot
          temp1= temp1 - dmu(j)*dplz(3,j,im)*dplz(3,j,im2)*dz(j)
          temp2=temp2 +(dro(j)*dplz(3,j,im)*dplz(3,j,im2)/ak(im)/ak(im2)
     &     - dmu(j)*dplz(6,j,im)*dplz(6,j,im2)/ak(im)/ak(im2))
     &        * dz(j)
  42    continue
        cp1(im,im2,ix,iy) = temp1*ak(im)/ak(im2)
        cp1(im2,im,ix,iy) = temp1*ak(im2)/ak(im) 
        cp2(im,im2,ix,iy) = temp2*ak(im)
        cp2(im2,im,ix,iy) = temp2*ak(im2)
        dc = -2.*(cp2(im,im2,ix,iy)/ak(im2)+cp1(im,im2,ix,iy))
  40  continue
  41  continue
c
      if (ix.eq.25.and.iy.eq.25) then
      write (in8,*) 'eigint',ix,iy
      do 130 i=1,nmm    
        do 130 j=1,nmm    
           write (in8,*) i,j,cp1(i,j,ix,iy),cp2(i,j,ix,iy)
 130  continue
      do 131 i=1,nmr    
        do 131 j=1,nmr    
           write (in8,*) i,j,cp3(i,j,ix,iy),cp4(i,j,ix,iy)
 131  continue
      endif
c
      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine elcoef(ix,iy,dmu,dla,dro)
c
c       calculate the elastic coefficients dmu, dla and dro as a function
c       of depth at the position (ix,iy)
c       with 3 possible methods:
c  celbet= 'iso': isotropic anomalies, absolute variations for rho, Vs
c                    and Vp are specified separately.
c  celbet= 'bet': isotropic anomalies, absolute values for Vs specified.
c                       Vp and rho calcualted from Vs anomalies.
c  celbet= 'rel': isotropic anomalies, relative variations for Vs specified.
c                       Vp and rho calcualted from Vs anomalies.
c
c  delbet: a value which can be used to modify the overall degree of
c          heterogeneity in the model without making a new file.
c          for example '-1' will change all your fast anomalies into
c          slow ones and vice-versa.
c
      include "couplage.inc"

      real dmu(nptz),dla(nptz),dro(nptz)

      om2 = 2.*3.14159/perio
      om2 = om2*om2
c
c......calculate the anomalies in mu, lambda and rho
c                 with three possible methods for isotropic anomalies

      if (celbet.eq.'iso') then
c        calculate directly from vs,vp and rho
        do j=ntop,nbot
          dbeta = delbet*grid(1,ix,iy,j)
          dalfa = delbet*grid(2,ix,iy,j)
          dro(j)= delbet*grid(3,ix,iy,j)
          dmu(j) = (ro(j)+dro(j))*(sqrt(al(j)/ro(j))+dbeta)**2 - al(j)
          dla(j) = (ro(j)+dro(j))*(sqrt(aa(j)/ro(j))+dalfa)**2
     &                    - aa(j) - 2.*dmu(j)
          dro(j) = om2*dro(j)
        enddo
      endif

      if (celbet.eq.'bet'.or.celbet.eq.'rel') then
c......  calculate the anomalies in mu, lambda and rho
c        from the anomalies in beta
c        with the hypotheses that anomalies are related to
c        temperature variations and elements in Anderson
c              "theory of the Earth"page 110.
        do j=ntop,nbot
c                  calculate relative beta anomaly
          dbeta = delbet*grid(1,ix,iy,j)
          if (celbet.eq.'bet') then
            if (al(j).eq.0.) stop 'no ABS model in water'
            dbeta=dbeta/sqrt(al(j)/ro(j))
          endif
          dmurel = 12./5.*dbeta
          dmu(j) = al(j)*dmurel
          dla(j) = af(j)*dmurel
          dro(j) = om2*ro(j)*dmurel/6.
        enddo
      endif
c
c     write out all the elements at one location and one depth
      if (ix.eq.25.and.iy.eq.25) then
      write (in8,*) 'elastic coefficients at 25 25 '
      k=30
      write (in8,*) dmu,dla,dro
      endif
      
      return
      end
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
      subroutine hetero (in2)
c      reads the heterogeneous model on unit in2

      include 'couplage.inc'

      real agrid(maxnz)

      read (in2,*)
      read (in2,*) nnx,nny,nnz
      read (in2,*) ddx,ddy,ddz
c
c...... index of the first and last points in the eigenfunctions
c      hmin: depth of heterogeneity with respect to the reference
c         level (i.e. top of water if indepm=0, bottom otherwise)
      hmax=hmin+(nnz-1)*ddz

      ntop  = 1
      nbot = 1
      dowhile (hh(ntop).lt.hmin)
        ntop=ntop+1
      enddo
      dowhile (hh(nbot+1).lt.hmax)
        nbot=nbot+1
      enddo
      write (in8,*) 'ntop nbot',ntop,nbot
c
c
c..........calculate the dz for the vertical integration
      dz(ntop) = (hh(ntop+1)-hh(ntop))/2.
      do 30 j =ntop+1,nbot-1
        dz(j) = (hh(j+1)-hh(j-1))/2.
  30  continue
      dz(nbot) = (hh(nbot)-hh(nbot-1))/2.

c......... read the model at different x-y positions and interpolate
c                  to the depths values of the eigenfunctions.
c                  uses the same index i as the eigenfunctions
c
c   if celbet= 'bet' or 'rel'       read the Vs model only
c   if celbet= 'iso'       read Vs , Vp and rho

      if (celbet.eq.'bet'.or.celbet.eq.'rel') nloop=1
      if (celbet.eq.'iso') nloop=3

      do iloop=1,nloop
        do ix=1,nnx
          do iy=1,nny
            read (in2,'(8f10.5)') (agrid(iz),iz=1,nnz)
            do i=ntop,nbot
c             j is the index of the heterogenous model close to the
c                 depth of the eigenfunction
              j=int((hh(i)-hmin)/ddz)+1
              grid(iloop,ix,iy,i)=agrid(j)+
     &            (agrid(j+1)-agrid(j))/ddz*(hh(i)-hmin-(j-1)*ddz)
            enddo
          enddo
        enddo
      enddo

      do j=ntop,nbot
      write (in8,'(10e12.3)')
     &grid(1,20,40,j),grid(2,20,40,j),grid(3,20,40,j),ro(j),al(j),hh(j),
     &     dplz(1,j,1),dplz(3,j,3)
      enddo

      return
      end
